package extmedias

import (
	"database/sql"
	"fmt"
	"net/http"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/movies"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"

	"github.com/Sirupsen/logrus"
	polochon "github.com/odwrtw/polochon/lib"
	traktExplorer "github.com/odwrtw/polochon/modules/trakttv"
	ytsExplorer "github.com/odwrtw/polochon/modules/yts"
)

// GetMediaIDs will get some media IDs
func GetMediaIDs(env *web.Env, mediaType string, source string, category string, force bool) ([]string, error) {
	log := env.Log.WithFields(logrus.Fields{
		"mediaType": mediaType,
		"source":    source,
		"category":  category,
		"function":  "extmedias.GetMediaIds",
	})

	var err error
	media, err := Get(env.Database, mediaType, source, category)
	switch err {
	case nil:
		log.Debug("medias found in database")
		if !force {
			log.Debug("returning medias from db")
			return media.IDs, nil
		}
	case sql.ErrNoRows:
		log.Debug("medias not found in database")
	default:
		// Unexpected error
		return nil, err
	}

	explorer, err := NewExplorer(env, source)
	if err != nil {
		return nil, err
	}

	var ids []string
	movies, err := explorer.GetMovieList(polochon.ExploreByRate, log)
	for _, movie := range movies {
		ids = append(ids, movie.ImdbID)
	}

	log.Debugf("got %d medias from %s", len(ids), source)

	media = &Media{
		Type:     mediaType,
		Source:   source,
		Category: category,
		IDs:      ids,
	}

	log.Debugf("Inserting int to DB %+v", media)
	err = media.Upsert(env.Database)
	if err != nil {
		return nil, err
	}

	log.Debug("medias updated in database")

	return ids, nil
}

// GetMedias get some movies
func GetMedias(env *web.Env, source string, category string, force bool) ([]*movies.Movie, error) {
	movieIds, err := GetMediaIDs(env, "movie", source, category, force)
	if err != nil {
		return nil, err
	}

	movieList := []*movies.Movie{}
	for _, id := range movieIds {
		movie := movies.New(id)
		err := movie.GetDetails(env, force)
		if err != nil {
			env.Log.Error(err)
			continue
		}
		err = movie.GetTorrents(env, force)
		if err != nil {
			env.Log.Error(err)
			continue
		}
		movieList = append(movieList, movie)
	}
	return movieList, nil
}

// Explore will explore some movies
func Explore(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	err := r.ParseForm()
	if err != nil {
		return err
	}

	source := r.FormValue("source")
	// Default source
	if source == "" {
		source = "yts"
	}

	category := r.FormValue("category")
	// Default category
	if category == "" {
		category = "popular"
	}

	// Get the medias without trying to refresh them
	movies, err := GetMedias(env, source, category, false)
	if err != nil {
		return err
	}

	return env.RenderJSON(w, movies)
}

// MediaSources represents the implemented media sources
var MediaSources = []string{
	"trakttv",
	"yts",
}

// Refresh will refresh the movie list
func Refresh(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	env.Log.Debugf("refreshing infos ...")
	source := r.FormValue("source")
	if source == "" {
		source = "yts"
	}

	category := r.FormValue("category")
	if category == "" {
		category = "popular"
	}

	// We'll refresh the medias for each sources
	for _, source := range MediaSources {
		env.Log.Debugf("refreshing %s", source)
		// GetMedias and refresh them
		_, err := GetMedias(env, source, category, true)
		if err != nil {
			return err
		}
	}

	return env.RenderJSON(w, map[string]string{"message": "Refresh is done"})
}

// NewExplorer returns a polochon.Explorer
func NewExplorer(env *web.Env, source string) (polochon.Explorer, error) {
	switch source {
	case "trakttv":
		return traktExplorer.NewExplorer(&traktExplorer.Params{
			ClientID: env.Config.TraktTVClientID,
		})
	case "yts":
		return ytsExplorer.NewExplorer()
	default:
		return nil, fmt.Errorf("unknown explorer")
	}
}
